import math

# const
file_spl_mic = "mems_mic_real_amplitud.mic"
file_phase_mic = "mems_mic_phase_test.mic"

def read_data_file(file_name):
  error = 0
  freq_values=[]
  voltage_value=[]
  file_data = open(file_name, 'r')
  frequency = ""
  voltage_Vpp = ""
  freq_bool = 1
  i = 0
  
  for data in file_data:
    if data[0] != '#':
        try:
            for symbol in data:
                if symbol == '\t': freq_bool = 0
                if freq_bool == 1: frequency = frequency + symbol
                if freq_bool == 0: voltage_Vpp = voltage_Vpp + symbol
            freq_values.append(float(frequency))
            voltage_value.append(float(voltage_Vpp))
        except:
            print("Error read file " + file_name + " in line " + str(i + 1))
            error = 0
            break
        frequency = ""
        voltage_Vpp = ""
        freq_bool = 1
        i = i + 1
  file_data.close()
  
  return error, freq_values, voltage_value

##########################################################################################
# Calculate Pa
##########################################################################################

freq_mic = read_data_file(file_spl_mic)[1]
spl_mic_dB = read_data_file(file_spl_mic)[2]
spl_mic_Pa = []
spl_mic = []
phase_grad = []

# Checking files for errors
# Calculate delta SPL
# Creating file delta SPL
file_phase = open(file_phase_mic, 'w')
file_phase.write("#Freq, Hz	Phase, grad.\n")

# SPL min and SPL max in dB
spl_min_dB = 10000
spl_max_dB = -10000
i = 0
for data in freq_mic:
	if (spl_min_dB > spl_mic_dB[i]): spl_min_dB = spl_mic_dB[i]
	if (spl_max_dB < spl_mic_dB[i]): spl_max_dB = spl_mic_dB[i]
	i = i + 1
spl_min_Pa = 0.00002 * math.pow(10, spl_min_dB/20.0) 
spl_max_Pa = 0.00002 * math.pow(10, spl_max_dB/20.0)
print ("spl min = " + str(spl_min_dB) + " dB, " + str(spl_min_Pa) + " Pa")
print ("spl max = " + str(spl_max_dB) + " dB, " + str(spl_max_Pa) + " Pa")


# Convert data
# Convert dB to Pa (SPL Pa = 0.00002 * 10^( SPL dB / 20 ))
# Convert Pa to dB (SPL dB = 20 * log10( SPL Pa/ 0.00002 ))
i = 0
for data in freq_mic:
	spl_mic.append(0.00002 * math.pow(10, (spl_mic_dB[i] + math.fabs(spl_min_dB))/20.0))
	#print(str(freq_mic[i]) + '\t' + str(spl_mic[i]))	
	i = i + 1

# New array freqyency and amplitude
freq_max_point = 36000 # Hz
freq_step = 10 # Hz

freq_sum = freq_mic[0]
freq_new_array = []
spl_new_array = []
fn_sign = [] # 1 or -1
phase_shift = [] # 1, 2, 3, 4 ...
i = 0
while (i < freq_max_point/freq_step):
	freq_new_array.append(freq_sum)
	j = 0
	while (j+1 < len(freq_mic)):
		if (freq_sum >= freq_mic[j] and freq_sum <= freq_mic[j+1]):
			spl_new_array.append((freq_sum-freq_mic[j])*(spl_mic[j+1]-spl_mic[j])/(freq_mic[j+1]-freq_mic[j])+spl_mic[j])
			#print (str(freq_new_array[i]) + '\t' + str(spl_new_array[i]))
		j = j + 1
	# Phase transitions through zero (extremum points are entered manually)
	num_sgn = -1.0
	num_phase = 1.0
	#if(freq_sum > 0 and freq_sum < 1000):
	#	num_sgn = -1.0
	#	num_phase = 1.0
	#if(freq_sum >= 1000 and freq_sum < 6500):
	#	num_sgn = -1.0
	#	num_phase = 1.0
	#if(freq_sum >= 6700 and freq_sum < 27900):
	#	num_sgn = 1.0
	#	num_phase = -1.0*math.pi 
	#if(freq_sum >= 27900):
	#	num_sgn = -1.0
	#	num_phase = 6.0*math.pi 
	fn_sign.append(num_sgn)
	phase_shift.append(num_phase)
	
	freq_sum = freq_sum + freq_step
	i = i + 1

##########################################################################################
# Calculate the phase (Hilbert transform)
##########################################################################################
i = 0
for freq in freq_new_array:
	spl_sum = 0
	j = 0
	while (j < i):
		spl_sum = spl_sum + spl_new_array[j]/float(i-j)
		j = j + 1
	spl_sum = spl_sum/math.pi
	amp_p = math.sqrt(spl_new_array[i]*spl_new_array[i] + spl_sum*spl_sum)
	phase_grad.append(fn_sign[i]*math.acos(spl_new_array[i]/amp_p)*(phase_shift[i]*180.0)/math.pi)
	
	print(str('{:.2f}'.format(freq)) + '\t' + str('{:.2f}'.format(phase_grad[i])))
	i = i + 1

# Phase shift
phase_delta = 0
phase_zero = 0
i = 0
phase_grad_new = []
phase_grad_new = phase_grad
while (i < len(freq_new_array)):
	if(i < len(freq_new_array)-1 and math.fabs((3600+phase_grad[i+1])-(3600+phase_grad[i])) > 10.0):
		phase_delta = phase_delta + (3600+phase_grad[i+1])-(3600+phase_grad[i])
		if(i > 0): phase_grad_new[i] = phase_grad[i] + phase_delta + (3600+phase_grad[i-1])-(3600+phase_grad[i])
		#print(str('{:.2f}'.format(freq_new_array[i])) + '\t' + str('{:.2f}'.format(phase_delta)))
	phase_grad_new[i] = phase_grad[i] - phase_delta
	#print(str('{:.2f}'.format(freq)) + '\t' + str('{:.2f}'.format(phase_grad_new[i])))
	
	# We shift the phase relative to 0 deg.
	if(freq_new_array[i] == 1000.0): phase_zero = phase_grad[i]
	i = i + 1

# We shift the phase relative to 0 deg. on 1000 Hz
i = 0
for freq in freq_new_array:
	phase_grad[i] = phase_grad_new[i]
	phase_grad[i] = 4.5 + phase_grad[i] - phase_zero
	i = i + 1

# Write data in file
i = 0
for freq in freq_new_array:
	#print(str('{:.2f}'.format(freq)) + '\t' + str('{:.2f}'.format(phase_grad[i])))
	#print(str('{:.2f}'.format(freq_mic[i])) + '\t' + str(spl_new_array[i]))
	
	# Write file
	file_phase.write(str('{:.3f}'.format(freq)) + '\t' + str('{:.2f}'.format(phase_grad[i]))+ '\n')
	#file_phase.write(str(freq) + '\t' + str('{:.2f}'.format(spl_new_array[i]))+ '\n')
	
	i = i + 1
file_phase.close()
